@@ -1,7 +1,7 @@
 (* 
  * prooftree --- proof tree display for Proof General
  * 
- * Copyright (C) 2011 - 2013 Hendrik Tews
+ * Copyright (C) 2011 - 2016 Hendrik Tews
  * 
  * This file is part of "prooftree".
  * 
@@ -19,7 +19,7 @@
  * You should have received a copy of the GNU General Public License
  * along with "prooftree". If not, see <http://www.gnu.org/licenses/>.
  * 
- * $Id: draw_tree.ml,v 1.48 2013/03/28 08:02:00 tews Exp $
+ * $Id: draw_tree.ml,v 1.54 2016/01/23 12:57:13 tews Exp $
  *)
 
 
@@ -408,8 +408,10 @@ object
   *)
   method window_number : string
 
-  (** Set the content in the text buffer of this node window *)
-  method update_content : string -> unit
+  (** Update the content in the text buffer of this node window. The
+      argument is the updated {!proof_tree_element.sequent_text_history}.
+  *)
+  method update_content : string list -> unit
 
   (** Reconfigure and redraw the node window. Needs to be called when
       the configuration has been changed. Actually only the font of
@@ -421,7 +423,7 @@ object
       when the corresponding element in the proof-tree display is
       deleted.
   *)
-  method delete_non_sticky_node_window : unit
+  method delete_attached_node_window : unit
 end
 
 
@@ -475,6 +477,7 @@ object (self)
   *)
   val drawable = drawable
 
+                                (***************** inside proof_tree_element *)
   (** The width of this node alone in pixels. Set in the initializer
       of the heirs. *)
   val mutable width = 0
@@ -504,6 +507,7 @@ object (self)
   *)
   val mutable x_offset = 0
 
+                                (***************** inside proof_tree_element *)
   (** The height of this nodes subtree, counted in tree levels. At
       least 1, because this element occupies already some level. 
   *)
@@ -529,12 +533,32 @@ object (self)
   val mutable tree_layer = 
     (None : proof_tree_element abstract_tree_container option)
 
+  (** This field is really used only inside {!turnstile}. In a
+      turnstile element, it holds the list of all previous versions of
+      the sequent text without existential information, except for the
+      head, with contains the current sequent {b with} existential
+      info. For uniform treatment of external node windows, the
+      reference is also used for proof commands. There, it holds just
+      one element, the proof command with existentials info.
+
+      The existential info is omitted from old versions of the sequent
+      text, because this info is incorrect for sequents that get
+      updated. The problem is that the exisitentials change already
+      with {!Proof_tree.add_new_goal}, which happens long before
+      {!Proof_tree.update_sequent}. A fix for this would require a
+      protocol change, which is a bit too much for this little
+      feature.
+  *)
+  val mutable sequent_text_history = []
+
+
   (***************************************************************************)
   (***************************************************************************)
   (** {2 Accessors / Setters} *)
   (***************************************************************************)
   (***************************************************************************)
 
+                                (***************** inside proof_tree_element *)
   (** Accessor method of {!attribute: width}. *)
   method width = width
 
@@ -563,6 +587,7 @@ object (self)
   method selected b = selected <- b
 
 
+                                (***************** inside proof_tree_element *)
   (** Accessor method of {!attribute: existential_variables}. *)
   method existential_variables = existential_variables
 
@@ -595,12 +620,25 @@ object (self)
     assert(tree_layer = None);
     tree_layer <- Some tl
 
+  (** Make {!sequent_text_history} accessible for cloning and for
+      reattaching external node windows.
+  *)
+  method sequent_text_history = sequent_text_history
+
+  (** This method is only used inside {!turnstile} but declared here to
+      avoid downcasting during cloning. Set the sequent text history,
+      used when cloning. *)
+  method set_sequent_text_history history =
+    sequent_text_history <- history
+
+
   (***************************************************************************)
   (***************************************************************************)
   (** {2 Children Iterators} *)
   (***************************************************************************)
   (***************************************************************************)
 
+                                (***************** inside proof_tree_element *)
   (** General iterator for all children. [iter_children left y a f]
       successively computes the [left] and [y] value of each child and
       calls [f left y c a] for each child [c] (starting with the
@@ -639,6 +677,7 @@ object (self)
   (***************************************************************************)
   (***************************************************************************)
 
+                                (***************** inside proof_tree_element *)
   (** Compute the height of the subtree of this element in pixels. *)
   method subtree_height = 
     (subtree_levels - 1) * !current_config.level_distance + 
@@ -672,6 +711,7 @@ object (self)
 	(0, 0, None)
 	children 
     in
+                                (***************** inside proof_tree_element *)
     subtree_levels <- max_levels + 1;
     subtree_width <- children_width;
     x_offset <- 
@@ -711,6 +751,7 @@ object (self)
       (* this node's left side is right of the left margin of the first child *)
       first_child_offset <- 0;
     end;
+                                (***************** inside proof_tree_element *)
     (* The real condition for the next if is
      *   subtree_width - x_offset < width / 2
      * but it has rounding issues when width is odd.
@@ -741,6 +782,7 @@ object (self)
      *   subtree_levels;
      *)
 	
+                                (***************** inside proof_tree_element *)
   (** Do {!update_subtree_size} in this element and all parent
       elements up to the root of the tree.
   *)
@@ -773,6 +815,7 @@ object (self)
   (***************************************************************************)
   (***************************************************************************)
 
+                                (***************** inside proof_tree_element *)
   (** Computes the left offset of [child] relative to the bounding box
       of its parent, which must be this node. *)
   method child_offset child =
@@ -803,6 +846,7 @@ object (self)
 	in
 	(left_off, y_off)
 
+                                (***************** inside proof_tree_element *)
   (** Computes the bounding box (that is a 4-tuple [(x_low, x_high,
       y_low, y_high)]) relative to the upper-left corner of the
       complete display. 
@@ -836,6 +880,7 @@ object (self)
      float_of_int (y_u + top))
 
 
+                                (***************** inside proof_tree_element *)
   (** Computes the x-coordinate of this node. Argument [left] must be
       the x-coordinate of the left side of the bounding box of this
       node's subtree.
@@ -866,6 +911,7 @@ object (self)
   method virtual line_offset : float -> (int * int)
 
 
+                                (***************** inside proof_tree_element *)
   (** Draw the lines from this node to all its children. 
 
       @param left x-coordinate of the left side of the bounding box of
@@ -889,6 +935,7 @@ object (self)
 	restore_gc drawable gc_opt)
 
 
+                                (***************** inside proof_tree_element *)
   (** Draw this element's subtree given the left side of the bounding box
       and the y-coordinate of this node. This is the internal draw method 
       that iterates through the tree.
@@ -917,6 +964,7 @@ object (self)
       (fun left y child -> child#draw_subtree left y)
 
 
+                                (***************** inside proof_tree_element *)
   (** Draw this node's subtree given the left and top side of the
       bounding box. This is the external draw method that is called 
       from the outside for the root of the tree.
@@ -936,6 +984,7 @@ object (self)
   (***************************************************************************)
   (***************************************************************************)
 
+                                (***************** inside proof_tree_element *)
   (** Iterate over the proof tree to determine the node that contains
       the point [(bx, by)]. Returns [None] if there is no node that
       contains this point. (If [bx] and [by] are the coordinates of a
@@ -967,6 +1016,7 @@ object (self)
       None
 
 
+                                (***************** inside proof_tree_element *)
   (** Iterate over the proof tree to determine the node that contains
       the point [(bx, by)]. Returns [None] if there is no node that
       contains this point. This is the external version that is called
@@ -998,6 +1048,7 @@ object (self)
 	| Some p -> p#mark_branch f
 	| None -> ()
 
+                                (***************** inside proof_tree_element *)
   (** Mark this element as [CurrentNode] and all the parent nodes as
       [Current] branch, see {!branch_state_type}. Relies on the
       invariant that the parent of a [Current] element is also marked
@@ -1029,6 +1080,7 @@ object (self)
 	else false
       )
 
+                                (***************** inside proof_tree_element *)
   (** Mark this node as [Cheated] and mark all parents that have only
       [Cheated] children as [Cheated] as well, see
       {!branch_state_type}.
@@ -1057,6 +1109,7 @@ object (self)
 	  | Cheated -> assert false
       )
 
+                                (***************** inside proof_tree_element *)
   (** Remove the [Proved] or [Cheated] mark of this element and all
       parent elements until an [Unproven] or [Current] element is met,
       see {!branch_state_type}. 
@@ -1094,7 +1147,8 @@ object (self)
   (***************************************************************************)
   (***************************************************************************)
 
-  (** Return the displayed sequent text for turnstile elements, which
+                                (***************** inside proof_tree_element *)
+  (** Return the displayed text for proof-tree elements, which
       contains additional information about uninstantiated and
       partially instantiated existentials.
   *)
@@ -1128,10 +1182,11 @@ object (self)
   method delete_external_window win =
     external_windows <- List.filter (fun w -> w <> win) external_windows
 
+                                (***************** inside proof_tree_element *)
   (** Delete all non-sticky external node windows of this node.
   *)
   method delete_non_sticky_external_windows =
-    List.iter (fun w -> w#delete_non_sticky_node_window) external_windows
+    List.iter (fun w -> w#delete_attached_node_window) external_windows
 
   (** Propagate this nodes existentials to all its children. This
       method is not recursive. It is used during normal operation,
@@ -1150,18 +1205,24 @@ object (self)
     List.iter (fun c -> c#propagate_existentials) children
 
   (** Update the list of existential variables in displayed sequent
-      text in the whole subtree of this element. This needs to be
-      called when some existential got instantiated or when an undo
-      uninstantiates some existential.
+      text and proof commands in the whole subtree of this element.
+      This needs to be called when some existential got instantiated
+      or when an undo uninstantiates some existential.
   *)
   method update_existentials_info =
+    (match sequent_text_history with 
+      | [] -> ()
+      | _ :: history -> 
+	sequent_text_history <- self#displayed_text :: history
+    );
     (if external_windows <> [] && existential_variables <> [] 
      then
-	let new_text = self#displayed_text in
-	List.iter (fun ew -> ew#update_content new_text) external_windows
+	List.iter (fun ew -> ew#update_content sequent_text_history)
+	  external_windows
     );
     List.iter (fun c -> c#update_existentials_info) children	
 
+                                (***************** inside proof_tree_element *)
   (** Hook to be called when the list of children has been changed.
       Adjusts the relative layout information of this element and all its
       parents and (non-recursively) propagates the existentials to all
@@ -1200,12 +1261,15 @@ end
     It's value is arbitrary for cloned proof trees.
 *)
 class turnstile (drawable : better_drawable) 
-                undo_state sequent_id sequent_text =
+                undo_state sequent_id sequent_text_option =
 object (self)
   inherit proof_tree_element drawable undo_state sequent_id [] [] as super
 
   (** The pure sequent text. *)
-  val mutable sequent_text = (sequent_text : string)
+  val mutable sequent_text = 
+    match sequent_text_option with
+      | Some t -> t
+      | None -> "waiting for sequent text"
 
   (** Pango layout for rendering text. Only created if an ID for
       an external window must be put into the display.
@@ -1228,12 +1292,33 @@ object (self)
   *)
   method id = sequent_id
 
-  (** Update the sequent text. *)
-  method update_sequent new_text = 
+                                         (***************** inside turnstile *)
+  (** Update the sequent text to a new version. *)
+  method update_sequent new_text =
+    (match sequent_text_history with
+      | [] -> ()
+      | _ :: old -> sequent_text_history <- sequent_text :: old
+    );
     sequent_text <- new_text;
-    let new_text = self#displayed_text in
-    List.iter 
-      (fun ew -> ew#update_content new_text)
+    sequent_text_history <- self#displayed_text :: sequent_text_history;
+    List.iter (fun ew -> ew#update_content sequent_text_history)
+      external_windows
+
+  (** Restore the previous version of the sequent text. *)
+  method undo_update_sequent =
+    (match sequent_text_history with
+      | [] -> assert false
+      | _ :: [] ->
+	(* This happens when undoing the initial update sequent
+	 * command for additional subgoals.
+	 *)
+	sequent_text_history <- [];
+	sequent_text <- "no sequent text available"
+      | _ :: old :: rest ->
+	sequent_text <- old;
+	sequent_text_history <- self#displayed_text :: rest
+    );
+    List.iter (fun ew -> ew#update_content sequent_text_history)
       external_windows
 
   (** Return the pango layout object of {!layout}. Create one if there
@@ -1256,6 +1341,7 @@ object (self)
     layout <- None;
     super#configuration_updated
 
+                                         (***************** inside turnstile *)
   (** Draw the turnstile symbol for this sequent at the indicated
       coordinates.
   *)
@@ -1293,7 +1379,7 @@ object (self)
 	  layout
     )
 
-
+                                         (***************** inside turnstile *)
   (** Draw this turnstile node.
 
       @param left x-coordinate of the left side of the bounding box of
@@ -1328,6 +1414,7 @@ object (self)
       2 * !current_config.turnstile_radius +
       2 * !current_config.turnstile_line_width
 
+                                         (***************** inside turnstile *)
   initializer
     self#set_node_size;
     (* 
@@ -1335,6 +1422,11 @@ object (self)
      *   self#debug_name width height;
      *)
     self#update_subtree_size;
+    (match sequent_text_option with
+      | None -> ()
+      | Some _ -> 
+	sequent_text_history <- [self#displayed_text]
+    );
     ()
 
 end
@@ -1400,6 +1492,7 @@ object (self)
   (** Height (in pixels) of the rendered proof command text. *)
   val mutable layout_height = 0
 
+                              (***************** inside proof_command_length *)
   (** This is a [Proof_command] element, see {!node_kind}. *)
   method node_kind = Proof_command
 
@@ -1429,6 +1522,7 @@ object (self)
     layout_width <- w;
     layout_height <- h
 
+                              (***************** inside proof_command_length *)
   (** Set {!displayed_command}. Called from the initializer and when
       the configuration has been changed.
   *)
@@ -1463,6 +1557,7 @@ object (self)
     layout <- make_layout drawable_arg#pango_context;
     super#configuration_updated
 
+                              (***************** inside proof_command_length *)
   (** Override {!proof_tree_element.register_external_window} because
       the displayed proof command must be rerendered when an external
       window is registered.
@@ -1479,6 +1574,7 @@ object (self)
     super#delete_external_window win;
     self#render_proof_command
 
+                              (***************** inside proof_command_length *)
   (** Draw just this command node.
 
       @param left x-coordinate of the left side of the bounding box of
@@ -1513,6 +1609,7 @@ object (self)
       drawable#rectangle 
 	~x:(x - w/2) ~y:(y - h/2) ~width:w ~height:h ();
 
+                              (***************** inside proof_command_length *)
   (** Compute the line offsets for proof-command nodes, see
       {!proof_tree_element.line_offset}
   *)
@@ -1540,6 +1637,7 @@ object (self)
      *   self#debug_name w width height;
      *)
     self#update_subtree_size;
+    sequent_text_history <- [self#displayed_text];
     (* Printf.fprintf (debugc()) "INIT PC %s done\n%!" self#debug_name; *)
     ()
 
@@ -1587,7 +1685,10 @@ let rec clone_tree_node new_pc new_seq ex_hash
 	 (List.map (clone_existentials ex_hash) node#fresh_existentials)
 	 : proof_command :> proof_tree_element)
     | Turnstile -> 
-      (new_seq node#id node#content : turnstile :> proof_tree_element)
+      let ts = new_seq node#id (Some node#content)
+      in
+      ts#set_sequent_text_history node#sequent_text_history;
+      (ts : turnstile :> proof_tree_element)
   in
   if Some node = old_selected
   then cloned_selected := Some clone;
