@@ -1,7 +1,7 @@
 (* 
  * prooftree --- proof tree display for Proof General
  * 
- * Copyright (C) 2011 - 2013 Hendrik Tews
+ * Copyright (C) 2011 - 2016 Hendrik Tews
  * 
  * This file is part of "prooftree".
  * 
@@ -19,7 +19,7 @@
  * You should have received a copy of the GNU General Public License
  * along with "prooftree". If not, see <http://www.gnu.org/licenses/>.
  * 
- * $Id: ext_dialog.ml,v 1.15 2013/03/28 08:02:00 tews Exp $
+ * $Id: ext_dialog.ml,v 1.18 2016/01/23 12:57:14 tews Exp $
  *)
 
 (** The Existential Variable Dialog *)
@@ -436,11 +436,11 @@ object (self)
   method key_pressed_callback ev =
     match GdkEvent.Key.keyval ev with 
       | ks when 
-	  (ks = GdkKeysyms._Q or ks = GdkKeysyms._q) 
+	  (ks = GdkKeysyms._Q || ks = GdkKeysyms._q) 
 	  && (List.mem `CONTROL (GdkEvent.Key.state ev))
 	  -> 
 	exit 0
-      | ks when (ks = GdkKeysyms._Q or ks = GdkKeysyms._q)  -> 
+      | ks when (ks = GdkKeysyms._Q || ks = GdkKeysyms._q)  -> 
 	self#destroy (); true
       | ks when ks = GdkKeysyms._Up -> 
 	scroll_adjustment table_v_adjustment (-1); true
@@ -598,7 +598,7 @@ let make_ext_dialog proof_window proof_name =
   let show_current_button = GButton.button
     ~label:"Show current" ~packing:button_box#pack () in
   let close_button = GButton.button
-    ~label:"Close" ~packing:(button_box#pack ~from:`END) () in
+    ~stock:`CLOSE ~packing:(button_box#pack ~from:`END) () in
 
   let ext_window = 
     new existential_variable_window proof_window top_window 
