@@ -1,7 +1,7 @@
 (* 
  * prooftree --- proof tree display for Proof General
  * 
- * Copyright (C) 2011 - 2013 Hendrik Tews
+ * Copyright (C) 2011 - 2016 Hendrik Tews
  * 
  * This file is part of "prooftree".
  * 
@@ -19,7 +19,7 @@
  * You should have received a copy of the GNU General Public License
  * along with "prooftree". If not, see <http://www.gnu.org/licenses/>.
  * 
- * $Id: help_window.ml,v 1.17 2013/03/28 08:02:00 tews Exp $
+ * $Id: help_window.ml,v 1.24 2016/01/23 12:57:14 tews Exp $
  *)
 
 
@@ -64,6 +64,9 @@ let help_text =
   let bold_proof_tree = (Bold, "Prooftree") in
   let bold_proof_general = (Bold, "Proof General")
   in
+  (**************************************************************************)
+  (***************************** Colors *************************************)
+  (**************************************************************************)
   [(Heading, "Colors");
    (Default, "\n\nThe meaning of the colors in the proof tree is as follows: ");
    (Color !proved_complete_gdk_color, 
@@ -84,6 +87,9 @@ that nevertheless depend on some not (yet) instantiated existential variable \
    (Default, " parameters \
 can be changed in the configuration dialog.\n\
 \n");
+   (*************************************************************************)
+   (***************************** Navigation ********************************)
+   (*************************************************************************)
    (Heading, "Navigation");
    (Default, "\n\n\
 In addition to scroll bars and the usual keys one can move the proof \
@@ -95,6 +101,9 @@ opposite direction). After setting a negative value for ");
 the proof tree instead (i.e, the proof tree moves in the same \
 direction as the mouse).\n\
 \n");
+   (*************************************************************************)
+   (***************************** Sequent window ****************************)
+   (*************************************************************************)
    (Heading, "The sequent window and additional displays");
    (Default, "\n\n\
 The sequent display below the proof tree \
@@ -105,13 +114,18 @@ back to default behavior. The initial size of the sequent display can \
 be set in the configuration dialog. A value of 0 hides the sequent display.\n\
 \n\
 A double click or a shift-click displays any goal or proof command \
-in an additional \
-window. These additional windows are deleted when the main proof-tree \
-window disappears, unless their ");
-   (Italic, "Sticky");
-   (Default, " button is pressed.\n\
+in an additional window. These additional windows are automatically updated, \
+for instance, if an existential variable is instantiated. For additional \
+sequent displays one can browse the instantiation history of the sequent \
+using the forward and backward buttons. These additional windows can be ");
+   (Italic, "detached");
+   (Default, " from the proof tree. A detached \
+display is neither automatically updated nor automatically deleted.\n\
 \n\
 ");
+   (*************************************************************************)
+   (***************************** Tooltips **********************************)
+   (*************************************************************************)
    (Heading, "Tooltips");
    (Default, "\n\n\
 If turnstile tool tips are switched on, the complete sequent text is \
@@ -123,6 +137,9 @@ length at which truncation happens can be set in the configuration dialog. \
 Any truncated proof command is displayed in full length as tool tip if the \
 mouse stays long enough above it (and if command tool tips are enabled).\n\
 \n");
+   (*************************************************************************)
+   (***************************** Existentials ******************************)
+   (*************************************************************************)
    (Heading, "Existential variables");
    (Default, "\n\
 \n");
@@ -152,7 +169,20 @@ this variable ");
    (Background !existential_instantiate_gdk_color,
     "(with orange background, by default)");
    (Default, " in the proof-tree display.\n\
+\n\
+With ");
+   (Bold, "Coq");
+   (Default, " >= 8.5, existential variables are severely broken in ");
+   bold_proof_tree;
+   (Default, " because ");
+   (Bold, "Coq");
+   (Default, " does not provide the necessary information, see ");
+   (Bold, "Coq");
+   (Default, " bug 4504.\n\
 \n");
+   (*************************************************************************)
+   (***************************** Menus *************************************)
+   (*************************************************************************)
    (Heading, "Menus");
    (Default, "\n\n\
 The menu button opens the main menu. A right click opens the context menu, \
@@ -183,8 +213,10 @@ proof tree window. This cloned proof tree is not connected with ");
 \n\
 The ");
    (Italic, "Show current");
-   (Default, " menu item repositions the proof tree such that the \
-current proof goal is visible.\n\
+   (Default, " and ");
+   (Italic, "Show selected");
+   (Default, " menu items reposition the proof tree such that the \
+current proof goal or, respectively, the selected node is visible.\n\
 \n\
 The item ");
    (Italic, "Existentials");
@@ -192,18 +224,17 @@ The item ");
 \n\
 The ");
    (Italic, "Configuration");
-   (Default, " item displays the configuration dialog. Changing values \
-there does only take effect after the ");
-   (Italic, "Apply");
-   (Default, " or ");
-   (Italic, "OK");
-   (Default, " button has been pressed. The ");
+   (Default, " item displays the configuration dialog. Changes in this dialog \
+immediately take effect. In this dialog, the ");
    (Italic, "Save");
    (Default, " button stores the current configuration values \
               in the file ");
    (Italic, config_file_location);
    (Default, ", which overwrites the build-in default configuration \
-at start up.\n\
+at start up. The ");
+   (Italic, "Cancel");
+   (Default, " button closes the dialog and resets the configuration to \
+the state before starting the configuration dialog.\n\
 \n\
 The ");
    (Italic, "Exit");
@@ -215,6 +246,9 @@ The ");
    bold_proof_tree;
    (Default, ".)\n\
 \n");
+   (*************************************************************************)
+   (***************************** Customization *****************************)
+   (*************************************************************************)
    (Heading, "Customization");
    (Default, "\n\n\
 A major part of the proof visualization task is done by ");
@@ -259,6 +293,10 @@ let fill_help_buffer (buffer : GText.buffer) =
     help_text
 
 
+(** Flag for option [-help]. *)
+let start_help_dialog = ref false
+
+
 (** Create and display a new help window. This function creates a new
     {xref lablgtk class GWindow.dialog} that contains the formatted
     help text inside a {xref lablgtk class GText.view}.
@@ -271,7 +309,11 @@ let show_help_window () =
       ~resizable:true
       ()
   in
-  help_win#add_button "Close" `CLOSE;
+  let close_fun _ =
+    help_win#destroy ();
+    if !start_help_dialog then exit 0
+  in
+  help_win#add_button_stock `CLOSE `CLOSE;
   let _help_title =
     GMisc.label
       ~markup:"<big><b>Prooftree Help</b></big>"
@@ -291,8 +333,8 @@ let show_help_window () =
       ~packing:help_scrolling#add ()
   in
   fill_help_buffer help_view#buffer;
-  ignore(help_win#connect#destroy ~callback:(fun () -> help_win#destroy()));
-  ignore(help_win#connect#response ~callback:(fun _ -> help_win#destroy()));
+  ignore(help_win#connect#destroy ~callback:close_fun);
+  ignore(help_win#connect#response ~callback:close_fun);
   help_win#set_default_size ~width:400 ~height:300;
   (* help_win#set_default_size ~width:800 ~height:800; *)
   help_win#show ()
