@@ -1,7 +1,7 @@
 (* 
  * prooftree --- proof tree display for Proof General
  * 
- * Copyright (C) 2011 - 2013 Hendrik Tews
+ * Copyright (C) 2011 - 2016 Hendrik Tews
  * 
  * This file is part of "prooftree".
  * 
@@ -19,7 +19,7 @@
  * You should have received a copy of the GNU General Public License
  * along with "prooftree". If not, see <http://www.gnu.org/licenses/>.
  * 
- * $Id: input.ml,v 1.37 2013/03/28 08:02:00 tews Exp $
+ * $Id: input.ml,v 1.43 2016/01/23 12:57:14 tews Exp $
  *)
 
 
@@ -29,13 +29,23 @@
  *****************************************************************************)
 (** {2 Communition Protocol with Proof General}
 
-    The communication protocol with Proof General is almost one-way
-    only: Proof General sends display messages to Prooftree and
-    Prooftree never requests anything from Proof General. Only when
-    the proof-tree window of the current proof is closed, Prooftree
-    notifies Proof General. The communication protocol is designed
-    such that Prooftree always knows in advance how many bytes it has
-    to read until the end of a display message.
+    The communication protocol with Proof General is mostly one-way:
+    Proof General sends display messages to Prooftree and Prooftree
+    never requests information for the proof-tree display from Proof
+    General. Prooftree sends a notification to Proof General when the
+    proof-tree window is closed. It also sends proof commands to Proof
+    General on request of the user.
+
+    The communication protocol between Proof General and Prooftree is
+    split into two parts: The display messages, which are sent from
+    Proof General to Prooftree and the notification messages, which
+    are sent from Prooftree to Proof General.
+
+    {3 Display Messages}
+
+    The protocol for the display messages is designed such that
+    Prooftree always knows in advance how many bytes it has to read
+    until the end of a message.
 
     All display messages consist of 
     {ul
@@ -55,7 +65,7 @@
 
     In the following list 
     of commands, ``%d'' stands for a positive integer and %s for a string
-    which contains no white space. ``\{cheated|not-cheated\}'' denotes
+    which contains no white space. ``\{cheated | not-cheated\}'' denotes
     the alternative of either ``cheated'' or ``not-cheated''. An
     integer following the keyword state is a state number. An integer
     following some xxx-bytes denotes the number of bytes of the next
@@ -79,9 +89,9 @@
     the first message. 
     }
     {-  {v current-goals state %d current-sequent %s \
-    {cheated|not-cheated} {new-layer|current-layer} proof-name-bytes %d \
-     command-bytes %d sequent-text-bytes %d additional-id-bytes %d \
-     existential-bytes %d\n\
+    {cheated | not-cheated} {new-layer | current-layer} proof-name-bytes %d \
+    command-bytes %d sequent-text-bytes %d additional-id-bytes %d \
+    existential-bytes %d\n\
     <data-proof-name>\n\
     <data-command>\n\
     <data-current-sequent>\n\
@@ -162,7 +172,7 @@
     {- Full name of the proof}
     }
     }
-    {- {v branch-finished state %d {cheated|not-cheated} \
+    {- {v branch-finished state %d {cheated | not-cheated} \
     proof-name-bytes %d command-bytes %d existential-bytes %d\n\
     <data-proof-name>\n\
     <data-command>\n\
@@ -207,6 +217,41 @@
     }
     }
     }
+
+    {3 Notification Messages}
+
+    The notification messages are sent from Prooftree to Proof General
+    as a consequence of certain user interactions. There are 3
+    different notification messages: for stopping the proof-tree
+    display, for undo and for sending proof scripts. All notification
+    messages are preceeded with a newline and the string 
+    [emacs exec:], followed by a space, for easy recognition in Proof
+    General.
+
+    The remaining part of the messages have the following format.
+
+    {ul
+    {- {v stop-displaying v}
+    
+    Prooftree sends this message to Proof General when the user closed
+    the proof-tree display of a proof currently under development.
+    Proof General then stops sending display commands for that proof.
+    }
+    {- {v undo %d v}
+
+    Prooftree sends the undo message, when the user selected an undo
+    for a certain sequent from the context menu. The integer is the
+    undo state number of the proof command child node of the selected
+    sequent. 
+    }
+    {- {v insert-proof-script %d\n<script data>\n v}
+
+    Prooftree sends this message when the user selected the Insert
+    command or Insert subproof items from the context menu. The
+    integer is the length of [<script data>] without the enclosing
+    newlines.
+    }
+    }
 *)
 
 (** Version number of the communication protocol described and
@@ -238,7 +283,7 @@ let protocol_version = 3
 *)
 
 (*****************************************************************************
-*****************************************************************************)
+ *****************************************************************************)
 
 
 open Configuration
@@ -308,15 +353,27 @@ let setup_input_backup_channel () =
   else if !current_config.copy_input = false &&
 	 !input_backup_filename = None
   then ()
-  else if !current_config.copy_input
-  then begin
-    input_backup_oc := Some(open_out !current_config.copy_input_file);
-    input_backup_filename := Some !current_config.copy_input_file;
-  end else begin
-    input_backup_oc := None;
-    input_backup_filename := None;
+  else begin
+    (match !input_backup_oc with
+      | None -> ()
+      | Some oc -> 
+	close_out oc;
+	input_backup_oc := None;
+	input_backup_filename := None;
+    );
+    if !current_config.copy_input
+    then begin
+      (try
+	 input_backup_oc := Some(open_out !current_config.copy_input_file);
+       with
+	 | Sys_error msg -> raise (Log_input_file_error msg)
+      );
+      input_backup_filename := Some !current_config.copy_input_file;
+    end else begin
+      input_backup_oc := None;
+      input_backup_filename := None;
+    end
   end
-    
 
 
 (** Input function for reading from the input channel. To make the
@@ -371,7 +428,7 @@ let rec get_string_cont s i len continuation_fn () =
   if i = len
   then begin
     (* Printf.fprintf (debugc()) "GS %d yields %s\n%!" len s; *)
-    continuation_fn s
+    continuation_fn (Bytes.to_string s)
   end
   else get_string_cont s i len continuation_fn ()
 
@@ -387,7 +444,7 @@ let rec get_string_cont s i len continuation_fn () =
 *)
 let get_string len continuation_fn =
   (* Printf.fprintf (debugc()) "GS %d enter\n%!" len; *)
-  let s = String.create len in
+  let s = Bytes.create len in
   get_string_cont s 0 len continuation_fn ()
 
 
@@ -426,8 +483,8 @@ let configure_prooftree proof_assistant pg_protocol_version =
     raise (Protocol_error 
 	     ((Printf.sprintf
 		 ("Communication protocol mismatch.\n"
-		  ^^ "Proof General uses version %02d\n"
-		  ^^ "but this version of Prooftree supports version %02d")
+		  ^^ "Proof General uses version %02d,\n"
+		  ^^ "but this version of Prooftree supports only version %02d.")
 		 pg_protocol_version protocol_version),
 	      None));
   configure_message_received := true
@@ -440,8 +497,8 @@ let parse_configure com_buf =
 
 (******************************************************************************
  ******************************************************************************
- * current-goals state %d current-sequent %s {cheated|not-cheated} \
- * {new-layer|current-layer}
+ * current-goals state %d current-sequent %s {cheated | not-cheated} \
+ * {new-layer | current-layer}
  * proof-name-bytes %d command-bytes %d sequent-text-bytes %d \
  * additional-id-bytes %d existential-bytes %d\n\
  * <data-proof-name>\n\
@@ -524,6 +581,15 @@ let parse_current_goals com_buf =
     (fun state current_sequent_id cheated_string layer_string 
       proof_name_bytes command_bytes sequent_text_bytes additional_id_bytes
       existential_bytes ->
+        (*
+         * Printf.fprintf (debugc())
+	 * 	       ("PCGs state %d current-sequent %s cheated %s layer %s "
+	 * 		^^ "proof-name-len %d command-len %d sequent-len %d "
+	 * 		^^ "id-len %d existential-len %d\n%!")
+	 * 	       state current_sequent_id cheated_string layer_string
+	 * 	       proof_name_bytes command_bytes sequent_text_bytes
+	 * 	       additional_id_bytes existential_bytes;
+         *)
 	get_string proof_name_bytes
 	  (fun proof_name ->
 	    get_string command_bytes
@@ -625,7 +691,7 @@ let parse_switch_goal com_buf =
 
 
 (******************************************************************************
- * branch-finished state %d {cheated|not-cheated} \
+ * branch-finished state %d {cheated | not-cheated} \
  * proof-name-bytes %d command-bytes %d existential-bytes %d\n\
  * <data-proof-name>\n\
  * <data-command>\n\
@@ -964,7 +1030,10 @@ let parse_input_callback_ex clist =
 let configuration_updated = setup_input_backup_channel
 
 (** Initialize this module and setup the GTK main loop callback for
-    [stdin]. Puts [stdin] into non-blocking mode.
+    [stdin]. Puts [stdin] into non-blocking mode. If
+    [setup_input_backup_channel] raises an [Sys_error] (because of an
+    invalid file name), this function is called a second time with
+    input logging disabled.
 *)
 let setup_input () =
   U.set_nonblock U.stdin;
