@@ -1,7 +1,7 @@
 (* 
  * prooftree --- proof tree display for Proof General
  * 
- * Copyright (C) 2011 - 2013 Hendrik Tews
+ * Copyright (C) 2011 - 2016 Hendrik Tews
  * 
  * This file is part of "prooftree".
  * 
@@ -19,7 +19,7 @@
  * You should have received a copy of the GNU General Public License
  * along with "prooftree". If not, see <http://www.gnu.org/licenses/>.
  * 
- * $Id: main.ml,v 1.19 2013/03/28 08:02:00 tews Exp $
+ * $Id: main.ml,v 1.23 2016/01/23 12:57:14 tews Exp $
  *)
 
 
@@ -35,13 +35,16 @@
 (** Main --- Argument parsing and program start *)
 
 
-(**/**)
-module U = Unix
-(**/**)
+open Util
+open Gtk_ext
 open Configuration
 open Help_window
 open Input
 
+(**/**)
+module U = Unix
+(**/**)
+
 
 (** Master hook to be called when the configuration has been updated.
     Reconfigures the complete program to reflect the changes in the
@@ -54,12 +57,6 @@ let configuration_updated () =
 
 let _ = configuration_updated_callback := configuration_updated
 
-(** Flag for option [-config]. *)
-let start_config_dialog = ref false
-
-(** Flag for option [-help]. *)
-let start_help_dialog = ref false
-
 (** Argument list for [Arg.parse] *)
 let arguments = Arg.align [
   ("-config", Arg.Set start_config_dialog,
@@ -101,7 +98,19 @@ let anon_fun s =
 let main () =
   try_load_config_file ();
   Arg.parse arguments anon_fun "prooftree";
-  setup_input();
+  (try
+     setup_input();
+   with
+     | Log_input_file_error msg ->
+       run_message_dialog
+	 (Printf.sprintf 
+	    "Prooftree startup error: Opening the input logging file \
+             failed with:\n    %s\nDisabeling input logging."
+	    msg)
+	 `WARNING;
+       update_configuration_record {!current_config with copy_input = false};
+       setup_input()
+  );
   Printf.printf 
     ("Prooftree version %s awaiting input on stdin.\n" ^^
 	"Entering LablGTK main loop ...\n\n%!")
