@@ -1,7 +1,7 @@
 (* 
  * prooftree --- proof tree display for Proof General
  * 
- * Copyright (C) 2011 - 2013 Hendrik Tews
+ * Copyright (C) 2011 - 2016 Hendrik Tews
  * 
  * This file is part of "prooftree".
  * 
@@ -19,7 +19,7 @@
  * You should have received a copy of the GNU General Public License
  * along with "prooftree". If not, see <http://www.gnu.org/licenses/>.
  * 
- * $Id: proof_window.ml,v 1.61 2013/03/28 08:02:01 tews Exp $
+ * $Id: proof_window.ml,v 1.67 2016/01/23 12:57:15 tews Exp $
  *)
 
 
@@ -139,7 +139,7 @@ object (self)
   val mutable selected_node = None
 
   (** List of all external non-orphaned node windows that belong to nodes 
-      in the current proof tree. This list if stored only for optimization.
+      in the current proof tree. This list is stored only for optimization.
   *)
   val mutable node_windows = []
 
@@ -323,7 +323,7 @@ object (self)
     current_node_offset_cache <- None;
     selected_node <- None;
     self#refresh_sequent_area;
-    List.iter (fun w -> w#delete_non_sticky_node_window) node_windows;
+    List.iter (fun w -> w#delete_attached_node_window) node_windows;
     assert(node_windows = []);
     self#clear_existential_dialog_for_reuse
 
@@ -438,7 +438,7 @@ object (self)
   method delete_proof_window =
     if destroy_in_progress = false then begin
       destroy_in_progress <- true;
-      List.iter (fun w -> w#delete_non_sticky_node_window) node_windows;
+      List.iter (fun w -> w#delete_attached_node_window) node_windows;
       self#destroy_existential_dialog;
       let self = (self :> proof_window) in
       cloned_proof_windows :=
@@ -467,11 +467,11 @@ object (self)
   method key_pressed_callback ev =
     match GdkEvent.Key.keyval ev with 
       | ks when 
-	  (ks = GdkKeysyms._Q or ks = GdkKeysyms._q) 
+	  (ks = GdkKeysyms._Q || ks = GdkKeysyms._q) 
 	  && (List.mem `CONTROL (GdkEvent.Key.state ev))
 	  -> 
 	exit 0
-      | ks when (ks = GdkKeysyms._Q or ks = GdkKeysyms._q)  -> 
+      | ks when (ks = GdkKeysyms._Q || ks = GdkKeysyms._q)  -> 
 	self#user_delete_proof_window (); true
       | ks when ks = GdkKeysyms._Left -> 
 	scroll_adjustment drawing_h_adjustment (-1); true
@@ -482,7 +482,7 @@ object (self)
       | ks when ks = GdkKeysyms._Down -> 
 	scroll_adjustment drawing_v_adjustment 1; true
 
-      | ks when (ks = GdkKeysyms._E or ks = GdkKeysyms._e)  -> 
+      | ks when (ks = GdkKeysyms._E || ks = GdkKeysyms._e)  -> 
       	self#show_existential_window (); true
 
       (* 
@@ -814,6 +814,12 @@ object (self)
 	then self#show_node current
 	else self#reposition_current_node ()
 
+  (** Make the selected node visible in a user friendly way. *)
+  method show_selected_node () =
+    match selected_node with
+      | None -> ()
+      | Some selected -> self#show_node selected
+
   (** Callback for the size_allocate signal of the drawing area.
       Position the proof tree in the viewport of the drawing area and
       redraw.
@@ -1306,8 +1312,8 @@ object (self)
    ***************************************************************************)
 
   (** Create a new turnstile node for sequents. *)
-  method new_turnstile undo_state sequent_id sequent_text =
-    new turnstile drawable undo_state sequent_id sequent_text
+  method new_turnstile undo_state sequent_id sequent_text_option =
+    new turnstile drawable undo_state sequent_id sequent_text_option
 
   (** Create a new proof command node. *)
   method new_proof_command undo_state command 
@@ -1386,7 +1392,7 @@ let rec make_proof_window name geometry_string =
   let button_h_box = GPack.hbox
     ~packing:button_box_align#add () in
   let dismiss_button = 
-    GButton.button ~label:"Dismiss" ~packing:button_h_box#pack ()
+    GButton.button ~stock:`CLOSE ~packing:button_h_box#pack ()
   in
   let message_label =
     GMisc.label ~selectable:true ~ellipsize:`END 
@@ -1415,6 +1421,7 @@ let rec make_proof_window name geometry_string =
   let main_menu_entries = 
     [`I("Clone", clone_fun);
      `I("Show current", proof_window#reposition_current_node);
+     `I("Show selected", proof_window#show_selected_node);
      `I("Existentials", proof_window#show_existential_window);
      `I("Configuration", show_config_window);
      `I("Help", show_help_window);
